"""init db

Revision ID: 54fee6a7ecd8
Revises: 
Create Date: 2023-02-28 22:26:57.307010

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '54fee6a7ecd8'
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    pass


def downgrade() -> None:
    pass
